import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { ProductModel, CartItem } from '../interface/product.interface';

/**
 * CartService - Demo Dependency Injection trong Angular
 * 
 * @Injectable decorator:
 * - Đánh dấu class này có thể được inject
 * - providedIn: 'root' = singleton cho toàn app
 * - Angular DI container sẽ tự động tạo và quản lý instance
 */
@Injectable({
  providedIn: 'root' // Singleton pattern - chỉ có 1 instance trong toàn app
})
export class CartService {
  // BehaviorSubject để theo dõi state changes
  private cartItemsSubject = new BehaviorSubject<CartItem[]>([]);
  private cartCountSubject = new BehaviorSubject<number>(0);

  // Expose as Observable để components có thể subscribe
  cartItems$: Observable<CartItem[]> = this.cartItemsSubject.asObservable();
  cartCount$: Observable<number> = this.cartCountSubject.asObservable();

  constructor() {
    console.log('🛒 CartService instance được tạo');
    // Load from localStorage if exists
    this.loadCartFromStorage();
  }

  /**
   * Thêm sản phẩm vào giỏ hàng
   */
  addToCart(product: ProductModel, quantity: number = 1): void {
    const currentItems = this.cartItemsSubject.value;
    const existingItemIndex = currentItems.findIndex(item => item.product.id === product.id);

    let updatedItems: CartItem[];
    
    if (existingItemIndex >= 0) {
      // Sản phẩm đã có trong giỏ - tăng quantity
      updatedItems = currentItems.map((item, index) => 
        index === existingItemIndex 
          ? { ...item, quantity: item.quantity + quantity }
          : item
      );
    } else {
      // Thêm sản phẩm mới
      updatedItems = [...currentItems, { product, quantity }];
    }

    this.updateCart(updatedItems);
    console.log(`✅ Đã thêm ${product.name} vào giỏ hàng`);
  }

  /**
   * Xóa sản phẩm khỏi giỏ hàng
   */
  removeFromCart(productId: number): void {
    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.filter(item => item.product.id !== productId);
    this.updateCart(updatedItems);
    console.log(`🗑️ Đã xóa sản phẩm ID ${productId} khỏi giỏ hàng`);
  }

  /**
   * Cập nhật số lượng sản phẩm
   */
  updateQuantity(productId: number, quantity: number): void {
    if (quantity <= 0) {
      this.removeFromCart(productId);
      return;
    }

    const currentItems = this.cartItemsSubject.value;
    const updatedItems = currentItems.map(item => 
      item.product.id === productId 
        ? { ...item, quantity }
        : item
    );

    this.updateCart(updatedItems);
    console.log(`📝 Đã cập nhật số lượng sản phẩm ID ${productId} = ${quantity}`);
  }

  /**
   * Tính tổng tiền
   */
  calculateTotal(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + (item.product.price * item.quantity),
      0
    );
  }

  /**
   * Lấy tổng số sản phẩm trong giỏ
   */
  getTotalItems(): number {
    return this.cartItemsSubject.value.reduce(
      (total, item) => total + item.quantity,
      0
    );
  }

  /**
   * Xóa toàn bộ giỏ hàng
   */
  clearCart(): void {
    this.updateCart([]);
    console.log('🧹 Đã xóa toàn bộ giỏ hàng');
  }

  /**
   * Lấy danh sách items hiện tại
   */
  getCurrentItems(): CartItem[] {
    return this.cartItemsSubject.value;
  }

  /**
   * Private method để update cart và sync với localStorage
   */
  private updateCart(items: CartItem[]): void {
    this.cartItemsSubject.next(items);
    this.cartCountSubject.next(this.getTotalItems());
    this.saveCartToStorage(items);
  }

  /**
   * Lưu giỏ hàng vào localStorage
   */
  private saveCartToStorage(items: CartItem[]): void {
    try {
      localStorage.setItem('cart', JSON.stringify(items));
    } catch (error) {
      console.error('❌ Không thể lưu giỏ hàng vào localStorage:', error);
    }
  }

  /**
   * Load giỏ hàng từ localStorage
   */
  private loadCartFromStorage(): void {
    try {
      const savedCart = localStorage.getItem('cart');
      if (savedCart) {
        const items: CartItem[] = JSON.parse(savedCart);
        this.cartItemsSubject.next(items);
        this.cartCountSubject.next(this.getTotalItems());
        console.log('📦 Đã load giỏ hàng từ localStorage');
      }
    } catch (error) {
      console.error('❌ Không thể load giỏ hàng từ localStorage:', error);
    }
  }
}
