import { Component, OnInit } from '@angular/core';
import { CommonModule } from '@angular/common';
import { CartService } from '../../../shared/data-access/service/cart.service';
import { CartExtService } from '../../../shared/data-access/service/cart-ext.service';

/**
 * BadProductComponent - Demo cách KHÔNG NÊN làm (Tight Coupling)
 * Khởi tạo trực tiếp dependencies, khó test và maintain
 */
class BadProductComponent {
  private cartService: CartService;

  constructor() {
    // ❌ BAD: Tight coupling - tạo dependency trực tiếp
    this.cartService = new CartService();
    console.log('❌ BadProductComponent: Tạo CartService trực tiếp');
  }

  addToCart(productId: number): void {
    console.log('❌ BadProductComponent: Adding product với tight coupling');
    // Logic thêm sản phẩm...
  }

  // ❌ Vấn đề:
  // - Không thể dễ dàng thay đổi implementation
  // - Khó test (không mock được)
  // - Vi phạm Dependency Inversion Principle
  // - Mỗi component sẽ có instance riêng (không share state)
}

/**
 * GoodProductComponent - Demo cách ĐÚNG (Dependency Injection)
 * Inject dependencies thông qua constructor
 */
class GoodProductComponent {
  constructor(private cartService: CartService) {
    console.log('✅ GoodProductComponent: CartService được inject');
  }

  addToCart(productId: number): void {
    console.log('✅ GoodProductComponent: Adding product với DI');
    // Logic thêm sản phẩm...
  }

  // ✅ Ưu điểm:
  // - Loose coupling
  // - Dễ test (có thể mock service)
  // - Có thể dễ dàng thay đổi implementation
  // - Share state với các component khác
}

/**
 * DIComparisonComponent - Demo so sánh Tight Coupling vs Dependency Injection
 */
@Component({
  selector: 'app-di-comparison',
  standalone: true,
  imports: [CommonModule],
  templateUrl: './di-comparison.component.html',
  styleUrls: ['./di-comparison.component.scss']
})
export class DIComparisonComponent implements OnInit {
  // Demo examples
  badExample: string = '';
  goodExample: string = '';
  overrideExample: string = '';

  // Inject CartService để demo
  constructor(private cartService: CartService) {
    console.log('🎯 DIComparisonComponent: CartService injected');
  }

  ngOnInit(): void {
    this.setupExamples();
    this.demonstrateDI();
  }

  private setupExamples(): void {
    // Bad example code
    this.badExample = `
// ❌ CÁCH KHÔNG NÊN LÀM (Tight Coupling)
class ProductComponent {
  cartService: CartService;
  
  constructor() {
    // Tạo dependency trực tiếp
    this.cartService = new CartService();
  }
  
  addToCart(product: Product) {
    this.cartService.addToCart(product);
  }
}

// Vấn đề:
// - Không thể thay đổi implementation
// - Khó test (không mock được)
// - Mỗi component có instance riêng
// - Vi phạm SOLID principles
    `;

    // Good example code
    this.goodExample = `
// ✅ CÁCH ĐÚNG (Dependency Injection)
@Component({
  selector: 'app-product',
  // ...
})
export class ProductComponent {
  constructor(private cartService: CartService) {
    // Angular DI container tự động inject
  }
  
  addToCart(product: Product) {
    this.cartService.addToCart(product);
  }
}

// Ưu điểm:
// - Loose coupling
// - Dễ test và mock
// - Share state giữa components
// - Dễ thay đổi implementation
    `;

    // Override example
    this.overrideExample = `
// 🔄 OVERRIDE PROVIDER
@NgModule({
  providers: [
    {
      provide: CartService,
      useClass: CartExtService  // Thay thế implementation
    }
  ]
})
export class AppModule {}

// Hoặc trong @Injectable
@Injectable({
  providedIn: 'root',
  useClass: CartExtService
})
export class CartService {}
    `;
  }

  private demonstrateDI(): void {
    console.log('🔍 Demonstrating DI concepts...');
    
    // Demo 1: Service instance
    console.log('📦 Current CartService instance:', this.cartService);
    console.log('🔢 Cart total:', this.cartService.calculateTotal());
    
    // Demo 2: Service methods
    this.cartService.getCurrentItems().forEach(item => {
      console.log(`🛒 Item in cart: ${item.product.name} x ${item.quantity}`);
    });
  }

  /**
   * Demo manual instantiation (BAD practice)
   */
  demoManualInstantiation(): void {
    console.log('❌ Demo: Manual instantiation (BAD)');
    
    // Tạo instance mới - KHÔNG được share state
    // const manualCartService = new CartService(); // TypeScript error: constructor is private
    
    console.log('❌ Không thể tạo CartService manually - Constructor injection required');
  }

  /**
   * Demo service methods thông qua injected service
   */
  demoServiceMethods(): void {
    console.log('✅ Demo: Using injected service');
    
    // Sử dụng injected service
    const total = this.cartService.calculateTotal();
    const itemCount = this.cartService.getTotalItems();
    
    console.log('💰 Total from injected service:', total);
    console.log('📦 Item count from injected service:', itemCount);
  }

  /**
   * Demo provider override concept
   */
  demoProviderOverride(): void {
    console.log('🔄 Demo: Provider Override Concept');
    console.log('📝 Check console for detailed explanation');
    
    // Giải thích trong console
    console.log(`
🔧 Provider Override trong Angular DI:

1. Default Provider:
   @Injectable({ providedIn: 'root' })
   export class CartService { ... }

2. Override trong NgModule:
   @NgModule({
     providers: [
       { provide: CartService, useClass: CartExtService }
     ]
   })

3. Override trong Component:
   @Component({
     providers: [
       { provide: CartService, useClass: CartExtService }
     ]
   })

4. Factory Provider:
   { 
     provide: CartService, 
     useFactory: (http: HttpClient) => new CartApiService(http),
     deps: [HttpClient]
   }

5. Value Provider:
   { provide: API_URL, useValue: 'https://api.example.com' }
    `);
  }

  /**
   * Demo singleton pattern
   */
  demoSingleton(): void {
    console.log('🔗 Demo: Singleton Pattern in DI');
    
    // Injected service là singleton
    const service1 = this.cartService;
    
    // Nếu inject ở component khác cũng sẽ là cùng instance
    console.log('🎯 CartService instance:', service1);
    console.log('🔄 Same instance across components due to providedIn: "root"');
  }
}
