import { Component, OnInit, OnDestroy } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Subject, takeUntil } from 'rxjs';
import { ProductService } from '../../../shared/data-access/service/product.service';
import { CartService } from '../../../shared/data-access/service/cart.service';
import { ProductModel } from '../../../shared/data-access/interface/product.interface';

/**
 * ProductListComponent - Demo Constructor Injection
 * 
 * Inject dependencies:
 * - ProductService: để load danh sách sản phẩm
 * - CartService: để thêm sản phẩm vào giỏ hàng
 * 
 * Lưu ý về DI:
 * - Angular tự động inject instances thông qua constructor
 * - Không cần khởi tạo manually với 'new'
 * - Services được share giữa các components (singleton)
 */
@Component({
  selector: 'app-product-list',
  standalone: true,
  imports: [CommonModule],
  templateUrl: './product-list.component.html',
  styleUrls: ['./product-list.component.scss']
})
export class ProductListComponent implements OnInit, OnDestroy {
  products: ProductModel[] = [];
  loading = true;
  cartCount = 0;
  cartTotal = 0;
  addingToCart: { [productId: number]: boolean } = {};

  private destroy$ = new Subject<void>();

  /**
   * Constructor Injection
   * Angular DI container tự động inject các dependencies
   * 
   * @param productService - Service để quản lý sản phẩm
   * @param cartService - Service để quản lý giỏ hàng
   */
  constructor(
    private productService: ProductService,
    private cartService: CartService
  ) {
    console.log('🎯 ProductListComponent constructor called');
    console.log('📦 ProductService injected:', !!this.productService);
    console.log('🛒 CartService injected:', !!this.cartService);
  }

  ngOnInit(): void {
    console.log('🚀 ProductListComponent ngOnInit');
    
    // Load products sử dụng injected ProductService
    this.loadProducts();

    // Subscribe to cart changes sử dụng injected CartService
    this.subscribeToCartChanges();
  }

  ngOnDestroy(): void {
    this.destroy$.next();
    this.destroy$.complete();
  }

  /**
   * Load danh sách sản phẩm từ ProductService
   */
  private loadProducts(): void {
    console.log('📦 Loading products using injected ProductService...');
    
    this.productService.getAllProducts()
      .pipe(takeUntil(this.destroy$))
      .subscribe({
        next: (products: ProductModel[]) => {
          this.products = products;
          this.loading = false;
          console.log('✅ Products loaded:', products.length);
        },
        error: (error: any) => {
          console.error('❌ Error loading products:', error);
          this.loading = false;
        }
      });
  }

  /**
   * Subscribe to cart changes từ CartService
   */
  private subscribeToCartChanges(): void {
    // Subscribe to cart count
    this.cartService.cartCount$
      .pipe(takeUntil(this.destroy$))
      .subscribe((count: number) => {
        this.cartCount = count;
        console.log('🔄 Cart count updated:', count);
      });

    // Subscribe to cart items để tính total
    this.cartService.cartItems$
      .pipe(takeUntil(this.destroy$))
      .subscribe(() => {
        this.cartTotal = this.cartService.calculateTotal();
        console.log('💰 Cart total updated:', this.cartTotal);
      });
  }

  /**
   * Thêm sản phẩm vào giỏ hàng sử dụng injected CartService
   */
  addToCart(product: ProductModel): void {
    console.log('🛒 Adding product to cart:', product.name);
    
    // Set loading state
    this.addingToCart[product.id] = true;

    // Simulate some processing time
    setTimeout(() => {
      // Sử dụng injected CartService
      this.cartService.addToCart(product, 1);
      
      // Reset loading state
      this.addingToCart[product.id] = false;
      
      console.log('✅ Product added to cart successfully');
    }, 500);
  }

  /**
   * Handle image error
   */
  onImageError(event: any): void {
    event.target.src = 'https://picsum.photos/200/200?random=999';
  }
}
